@@ -36,14 +36,15 @@
 #include "Peer.h"
 #include "BtMessageDispatcher.h"
 #include "BtRequestFactory.h"
+#include "SocketBuffer.h"
 
 namespace aria2 {
 
-const std::string BtChokeMessage::NAME("choke");
+const char BtChokeMessage::NAME[] = "choke";
 
-BtChokeMessage::BtChokeMessage():ZeroBtMessage(ID, NAME) {}
+BtChokeMessage::BtChokeMessage():ZeroBtMessage{ID, NAME} {}
 
-SharedHandle<BtChokeMessage> BtChokeMessage::create
+std::unique_ptr<BtChokeMessage> BtChokeMessage::create
 (const unsigned char* data, size_t dataLength)
 {
   return ZeroBtMessage::create<BtChokeMessage>(data, dataLength);
@@ -64,10 +65,26 @@ bool BtChokeMessage::sendPredicate() const
   return !getPeer()->amChoking();
 }
 
-void BtChokeMessage::onSendComplete()
+namespace {
+struct ThisProgressUpdate : public ProgressUpdate {
+  ThisProgressUpdate(const std::shared_ptr<Peer>& peer,
+                     BtMessageDispatcher* disp)
+    : peer(peer), disp(disp) {}
+  virtual void update(size_t length, bool complete) CXX11_OVERRIDE
+  {
+    if(complete) {
+      peer->amChoking(true);
+      disp->doChokingAction();
+    }
+  }
+  std::shared_ptr<Peer> peer;
+  BtMessageDispatcher* disp;
+};
+} // namespace
+
+std::unique_ptr<ProgressUpdate> BtChokeMessage::getProgressUpdate()
 {
-  getPeer()->amChoking(true);
-  getBtMessageDispatcher()->doChokingAction();
+  return make_unique<ThisProgressUpdate>(getPeer(), getBtMessageDispatcher());
 }
 
 } // namespace aria2
