@@ -40,66 +40,66 @@
 #include "HttpConnection.h"
 #include "HttpRequest.h"
 #include "Segment.h"
-#include "Socket.h"
+#include "SocketCore.h"
 #include "prefs.h"
 #include "Option.h"
 #include "HttpResponse.h"
 #include "HttpHeader.h"
 #include "Range.h"
 #include "DownloadContext.h"
-#include "Decoder.h"
-#include "RequestGroupMan.h"
-#include "FileAllocationEntry.h"
-#include "CheckIntegrityEntry.h"
-#include "ServerStatMan.h"
 #include "Logger.h"
+#include "StreamFilter.h"
+#include "SinkStreamFilter.h"
+#include "util.h"
+#include "SocketRecvBuffer.h"
+
 namespace aria2 {
 
 HttpDownloadCommand::HttpDownloadCommand
 (cuid_t cuid,
- const SharedHandle<Request>& req,
- const SharedHandle<FileEntry>& fileEntry,
+ const std::shared_ptr<Request>& req,
+ const std::shared_ptr<FileEntry>& fileEntry,
  RequestGroup* requestGroup,
- const SharedHandle<HttpResponse>& httpResponse,
- const HttpConnectionHandle& httpConnection,
+ std::unique_ptr<HttpResponse> httpResponse,
+ const std::shared_ptr<HttpConnection>& httpConnection,
  DownloadEngine* e,
- const SocketHandle& socket)
-  :DownloadCommand(cuid, req, fileEntry, requestGroup, e, socket),
-   httpResponse_(httpResponse),
-   httpConnection_(httpConnection) {}
+ const std::shared_ptr<SocketCore>& socket)
+  : DownloadCommand(cuid, req, fileEntry, requestGroup, e, socket,
+                    httpConnection->getSocketRecvBuffer()),
+    httpResponse_(std::move(httpResponse)),
+    httpConnection_(httpConnection)
+{}
 
 HttpDownloadCommand::~HttpDownloadCommand() {}
 
 bool HttpDownloadCommand::prepareForNextSegment() {
   bool downloadFinished = getRequestGroup()->downloadFinished();
   if(getRequest()->isPipeliningEnabled() && !downloadFinished) {
-    HttpRequestCommand* command =
-      new HttpRequestCommand(getCuid(), getRequest(), getFileEntry(),
-                             getRequestGroup(), httpConnection_,
-                             getDownloadEngine(), getSocket());
+    auto command = make_unique<HttpRequestCommand>
+      (getCuid(), getRequest(), getFileEntry(),
+       getRequestGroup(), httpConnection_,
+       getDownloadEngine(), getSocket());
     // Set proxy request here. aria2 sends the HTTP request specialized for
     // proxy.
     if(resolveProxyMethod(getRequest()->getProtocol()) == V_GET) {
       command->setProxyRequest(createProxyRequest());
     }
-    getDownloadEngine()->addCommand(command);
+    getDownloadEngine()->addCommand(std::move(command));
     return true;
   } else {
+    const std::string& streamFilterName = getStreamFilter()->getName();
     if(getRequest()->isPipeliningEnabled() ||
        (getRequest()->isKeepAliveEnabled() &&
         (
-         // Make sure that all decoders are finished to pool socket
-         ((!getTransferEncodingDecoder().isNull() &&
-           getTransferEncodingDecoder()->finished()) ||
-          (getTransferEncodingDecoder().isNull() &&
-           !getContentEncodingDecoder().isNull() &&
-           getContentEncodingDecoder()->finished())) ||
+         // Make sure that all filters are finished to pool socket
+         (!util::endsWith(streamFilterName, SinkStreamFilter::NAME) &&
+          getStreamFilter()->finished()) ||
          getRequestEndOffset() ==
          getFileEntry()->gtoloff(getSegments().front()->getPositionToWrite())
          )
         )
        ) {
-      // TODO What if server sends EOF when _contentEncodingDecoder is
+      // TODO What if server sends EOF when non-SinkStreamFilter is
       // used and server didn't send Connection: close? We end up to
       // pool terminated socket.  In HTTP/1.1, keep-alive is default,
       // so closing connection without Connection: close header means
@@ -115,15 +115,13 @@ bool HttpDownloadCommand::prepareForNextSegment() {
     if(!getRequest()->isPipeliningEnabled() &&
        getRequest()->isPipeliningHint() &&
        !downloadFinished) {
-      const SharedHandle<Segment>& segment = getSegments().front();
+      const std::shared_ptr<Segment>& segment = getSegments().front();
 
-      off_t lastOffset =getFileEntry()->gtoloff
-        (std::min(static_cast<off_t>
-                  (segment->getPosition()+segment->getLength()),
+      int64_t lastOffset =getFileEntry()->gtoloff
+        (std::min(segment->getPosition()+segment->getLength(),
                   getFileEntry()->getLastOffset()));
-      
-      if(lastOffset ==
-         httpResponse_->getHttpHeader()->getRange()->getEndByte()+1) {
+      Range range = httpResponse_->getHttpHeader()->getRange();
+      if(lastOffset == range.endByte + 1) {
         return prepareForRetry(0);
       }
     }
@@ -131,9 +129,14 @@ bool HttpDownloadCommand::prepareForNextSegment() {
   }
 }
 
-off_t HttpDownloadCommand::getRequestEndOffset() const
+int64_t HttpDownloadCommand::getRequestEndOffset() const
 {
-  return httpResponse_->getHttpHeader()->getRange()->getEndByte()+1;
+  int64_t endByte = httpResponse_->getHttpHeader()->getRange().endByte;
+  if(endByte > 0) {
+    return endByte+1;
+  } else {
+    return endByte;
+  }
 }
 
 } // namespace aria2
