@@ -36,50 +36,62 @@
 
 #include <algorithm>
 
+#include "Logger.h"
 #include "LogFactory.h"
+#include "fmt.h"
 
 namespace aria2 {
 
-UTMetadataRequestTracker::UTMetadataRequestTracker():
-  _logger(LogFactory::getInstance()) {}
+UTMetadataRequestTracker::UTMetadataRequestTracker() {}
 
 void UTMetadataRequestTracker::add(size_t index)
 {
-  _trackedRequests.push_back(RequestEntry(index));
+  trackedRequests_.push_back(RequestEntry(index));
 }
 
 bool UTMetadataRequestTracker::tracks(size_t index)
 {
-  return std::find(_trackedRequests.begin(), _trackedRequests.end(),
-                   RequestEntry(index)) != _trackedRequests.end();
+  return std::find(trackedRequests_.begin(), trackedRequests_.end(),
+                   RequestEntry(index)) != trackedRequests_.end();
 }
 
 void UTMetadataRequestTracker::remove(size_t index)
 {
-  std::vector<RequestEntry>::iterator i =
-    std::find(_trackedRequests.begin(), _trackedRequests.end(),
-              RequestEntry(index));
-  if(i != _trackedRequests.end()) {
-    _trackedRequests.erase(i);
+  auto i = std::find(trackedRequests_.begin(), trackedRequests_.end(),
+                     RequestEntry(index));
+  if(i != trackedRequests_.end()) {
+    trackedRequests_.erase(i);
   }
 }
 
+namespace {
+struct RemoveTimeoutEntry {
+  RemoveTimeoutEntry(std::vector<size_t>& indexes)
+    : indexes(indexes)
+  {}
+  bool operator()(const UTMetadataRequestTracker::RequestEntry& ent) const
+  {
+    if(ent.elapsed(TIMEOUT)) {
+      A2_LOG_DEBUG(fmt("ut_metadata request timeout. index=%lu",
+                       static_cast<unsigned long>(ent.index_)));
+      indexes.push_back(ent.index_);
+      return true;
+    } else {
+      return false;
+    }
+  }
+  std::vector<size_t>& indexes;
+  static const time_t TIMEOUT = 20;
+};
+} // namespace
+
 std::vector<size_t> UTMetadataRequestTracker::removeTimeoutEntry()
 {
   std::vector<size_t> indexes;
-  const time_t TIMEOUT = 20;
-  for(std::vector<RequestEntry>::iterator i = _trackedRequests.begin();
-      i != _trackedRequests.end();) {
-    if((*i).elapsed(TIMEOUT)) {
-      LogFactory::getInstance()->debug
-        ("ut_metadata request timeout. index=%lu",
-         static_cast<unsigned long>((*i)._index));
-      indexes.push_back((*i)._index);
-      i = _trackedRequests.erase(i);
-    } else {
-      ++i;
-    }
-  }
+  trackedRequests_.erase(std::remove_if(trackedRequests_.begin(),
+                                        trackedRequests_.end(),
+                                        RemoveTimeoutEntry(indexes)),
+                         trackedRequests_.end());
   return indexes;
 }
 
@@ -96,9 +108,8 @@ size_t UTMetadataRequestTracker::avail() const
 std::vector<size_t> UTMetadataRequestTracker::getAllTrackedIndex() const
 {
   std::vector<size_t> indexes;
-  for(std::vector<RequestEntry>::const_iterator i = _trackedRequests.begin();
-      i != _trackedRequests.end(); ++i) {
-    indexes.push_back((*i)._index);
+  for(auto i = trackedRequests_.begin(), eoi = trackedRequests_.end(); i != eoi; ++i) {
+    indexes.push_back((*i).index_);
   }
   return indexes;
 }
