@@ -34,7 +34,6 @@
 /* copyright --> */
 #include "download_helper.h"
 
-#include <iostream>
 #include <algorithm>
 #include <sstream>
 
@@ -43,8 +42,7 @@
 #include "prefs.h"
 #include "Metalink2RequestGroup.h"
 #include "ProtocolDetector.h"
-#include "ParameterizedStringParser.h"
-#include "PStringBuildVisitor.h"
+#include "paramed_string.h"
 #include "UriListParser.h"
 #include "DownloadContext.h"
 #include "RecoverableException.h"
@@ -61,134 +59,24 @@
 #include "a2functional.h"
 #include "ByteArrayDiskWriterFactory.h"
 #include "MetadataInfo.h"
+#include "OptionParser.h"
+#include "SegList.h"
 #ifdef ENABLE_BITTORRENT
-# include "bittorrent_helper.h"
-# include "BtConstants.h"
-# include "UTMetadataPostDownloadHandler.h"
+#  include "bittorrent_helper.h"
+#  include "BtConstants.h"
+#  include "UTMetadataPostDownloadHandler.h"
+#  include "ValueBaseBencodeParser.h"
 #endif // ENABLE_BITTORRENT
 
 namespace aria2 {
 
-const std::set<std::string>& listRequestOptions()
-{
-  static const std::string REQUEST_OPTIONS[] = {
-    PREF_DIR,
-    PREF_CHECK_INTEGRITY,
-    PREF_CONTINUE,
-    PREF_ALL_PROXY,
-    PREF_ALL_PROXY_USER,
-    PREF_ALL_PROXY_PASSWD,
-    PREF_CONNECT_TIMEOUT,
-    PREF_DRY_RUN,
-    PREF_LOWEST_SPEED_LIMIT,
-    PREF_MAX_FILE_NOT_FOUND,
-    PREF_MAX_TRIES,
-    PREF_NO_PROXY,
-    PREF_OUT,
-    PREF_PROXY_METHOD,
-    PREF_REMOTE_TIME,
-    PREF_SPLIT,
-    PREF_TIMEOUT,
-    PREF_HTTP_AUTH_CHALLENGE,
-    PREF_HTTP_NO_CACHE,
-    PREF_HTTP_USER,
-    PREF_HTTP_PASSWD,
-    PREF_HTTP_PROXY,
-    PREF_HTTP_PROXY_USER,
-    PREF_HTTP_PROXY_PASSWD,
-    PREF_HTTPS_PROXY,
-    PREF_HTTPS_PROXY_USER,
-    PREF_HTTPS_PROXY_PASSWD,
-    PREF_REFERER,
-    PREF_ENABLE_HTTP_KEEP_ALIVE,
-    PREF_ENABLE_HTTP_PIPELINING,
-    PREF_HEADER,
-    PREF_USE_HEAD,
-    PREF_USER_AGENT,
-    PREF_FTP_USER,
-    PREF_FTP_PASSWD,
-    PREF_FTP_PASV,
-    PREF_FTP_PROXY,
-    PREF_FTP_PROXY_USER,
-    PREF_FTP_PROXY_PASSWD,
-    PREF_FTP_TYPE,
-    PREF_FTP_REUSE_CONNECTION,
-    PREF_NO_NETRC,
-    PREF_REUSE_URI,
-    PREF_SELECT_FILE,
-    PREF_BT_ENABLE_LPD,
-    PREF_BT_EXTERNAL_IP,
-    PREF_BT_HASH_CHECK_SEED,
-    PREF_BT_MAX_OPEN_FILES,
-    PREF_BT_MAX_PEERS,
-    PREF_BT_METADATA_ONLY,
-    PREF_BT_MIN_CRYPTO_LEVEL,
-    PREF_BT_PRIORITIZE_PIECE,
-    PREF_BT_REQUIRE_CRYPTO,
-    PREF_BT_REQUEST_PEER_SPEED_LIMIT,
-    PREF_BT_SAVE_METADATA,
-    PREF_BT_SEED_UNVERIFIED,
-    PREF_BT_STOP_TIMEOUT,
-    PREF_BT_TRACKER_INTERVAL,
-    PREF_BT_TRACKER_TIMEOUT,
-    PREF_BT_TRACKER_CONNECT_TIMEOUT,
-    PREF_ENABLE_PEER_EXCHANGE,
-    PREF_FOLLOW_TORRENT,
-    PREF_INDEX_OUT,
-    PREF_MAX_UPLOAD_LIMIT,
-    PREF_SEED_RATIO,
-    PREF_SEED_TIME,
-    PREF_FOLLOW_METALINK,
-    PREF_METALINK_SERVERS,
-    PREF_METALINK_LANGUAGE,
-    PREF_METALINK_LOCATION,
-    PREF_METALINK_OS,
-    PREF_METALINK_VERSION,
-    PREF_METALINK_PREFERRED_PROTOCOL,
-    PREF_METALINK_ENABLE_UNIQUE_PROTOCOL,
-    PREF_ALLOW_OVERWRITE,
-    PREF_ALLOW_PIECE_LENGTH_CHANGE,
-    PREF_ASYNC_DNS,
-    PREF_AUTO_FILE_RENAMING,
-    PREF_FILE_ALLOCATION,
-    PREF_MAX_DOWNLOAD_LIMIT,
-    PREF_NO_FILE_ALLOCATION_LIMIT,
-    PREF_PARAMETERIZED_URI,
-    PREF_REALTIME_CHUNK_CHECKSUM,
-    PREF_REMOVE_CONTROL_FILE,
-    PREF_ALWAYS_RESUME,
-    PREF_MAX_RESUME_FAILURE_TRIES,
-    PREF_HTTP_ACCEPT_GZIP,
-    PREF_MAX_CONNECTION_PER_SERVER,
-    PREF_MIN_SPLIT_SIZE,
-    PREF_CONDITIONAL_GET,
-    PREF_ENABLE_ASYNC_DNS6,
-    PREF_BT_TRACKER,
-    PREF_BT_EXCLUDE_TRACKER,
-    PREF_RETRY_WAIT,
-    PREF_METALINK_BASE_URI,
-    PREF_PAUSE,
-    PREF_STREAM_PIECE_SELECTOR,
-    PREF_HASH_CHECK_ONLY,
-    PREF_CHECKSUM,
-    PREF_PIECE_LENGTH
-  };
-  static std::set<std::string> requestOptions
-    (vbegin(REQUEST_OPTIONS), vend(REQUEST_OPTIONS));
-  return requestOptions;
-}
-
 namespace {
 void unfoldURI
 (std::vector<std::string>& result, const std::vector<std::string>& args)
 {
-  ParameterizedStringParser p;
-  PStringBuildVisitor v;
-  for(std::vector<std::string>::const_iterator itr = args.begin(),
-        eoi = args.end(); itr != eoi; ++itr) {
-    v.reset();
-    p.parse(*itr)->accept(v);
-    result.insert(result.end(), v.getURIs().begin(), v.getURIs().end()); 
+  for(const auto& i : args) {
+    paramed_string::expand(std::begin(i), std::end(i),
+                           std::back_inserter(result));
   }
 }
 } // namespace
@@ -203,63 +91,89 @@ void splitURI(std::vector<std::string>& result,
 {
   size_t numURIs = std::distance(begin, end);
   if(numURIs >= numSplit) {
-    result.insert(result.end(), begin, end);
+    result.insert(std::end(result), begin, end);
   } else if(numURIs > 0) {
     size_t num = std::min(numSplit/numURIs, maxIter);
     for(size_t i = 0; i < num; ++i) {
-      result.insert(result.end(), begin, end);
+      result.insert(std::end(result), begin, end);
     }
     if(num < maxIter) {
-      result.insert(result.end(), begin, begin+(numSplit%numURIs));
+      result.insert(std::end(result), begin, begin+(numSplit%numURIs));
     }
   }
 }
 } // namespace
 
 namespace {
-SharedHandle<RequestGroup> createRequestGroup
-(const SharedHandle<Option>& option, const std::vector<std::string>& uris,
+std::shared_ptr<GroupId> getGID(const std::shared_ptr<Option>& option)
+{
+  std::shared_ptr<GroupId> gid;
+  if(option->defined(PREF_GID)) {
+    a2_gid_t n;
+    if(GroupId::toNumericId(n, option->get(PREF_GID).c_str()) != 0) {
+      throw DL_ABORT_EX(fmt("%s is invalid for GID.",
+                            option->get(PREF_GID).c_str()));
+    }
+    gid = GroupId::import(n);
+    if(!gid) {
+      throw DL_ABORT_EX(fmt("GID %s is not unique.",
+                            option->get(PREF_GID).c_str()));
+    }
+  } else {
+    gid = GroupId::create();
+  }
+  return gid;
+}
+} // namespace
+
+namespace {
+std::shared_ptr<RequestGroup> createRequestGroup
+(const std::shared_ptr<Option>& optionTemplate,
+ const std::vector<std::string>& uris,
  bool useOutOption = false)
 {
-  SharedHandle<RequestGroup> rg(new RequestGroup(option));
-  SharedHandle<DownloadContext> dctx
-    (new DownloadContext
-     (option->getAsInt(PREF_PIECE_LENGTH),
-      0,
-      useOutOption&&!option->blank(PREF_OUT)?
-      util::applyDir(option->get(PREF_DIR), option->get(PREF_OUT)):A2STR::NIL));
+  auto option = util::copy(optionTemplate);
+  auto rg = std::make_shared<RequestGroup>(getGID(option), option);
+  auto dctx = std::make_shared<DownloadContext>
+    (option->getAsInt(PREF_PIECE_LENGTH),
+     0,
+     useOutOption&&!option->blank(PREF_OUT)?
+     util::applyDir(option->get(PREF_DIR),
+                    option->get(PREF_OUT)):A2STR::NIL);
   dctx->getFirstFileEntry()->setUris(uris);
   dctx->getFirstFileEntry()->setMaxConnectionPerServer
     (option->getAsInt(PREF_MAX_CONNECTION_PER_SERVER));
 #ifdef ENABLE_MESSAGE_DIGEST
   const std::string& checksum = option->get(PREF_CHECKSUM);
   if(!checksum.empty()) {
-    std::pair<std::string, std::string> p;
-    util::divide(p, checksum, '=');
-    util::lowercase(p.first);
-    util::lowercase(p.second);
-    dctx->setDigest(p.first, p.second);
+    auto p = util::divide(std::begin(checksum), std::end(checksum), '=');
+    std::string hashType(p.first.first, p.first.second);
+    std::string hexDigest(p.second.first, p.second.second);
+    util::lowercase(hashType);
+    dctx->setDigest(hashType,
+                    util::fromHex(std::begin(hexDigest), std::end(hexDigest)));
   }
 #endif // ENABLE_MESSAGE_DIGEST
   rg->setDownloadContext(dctx);
   rg->setPauseRequested(option->getAsBool(PREF_PAUSE));
-  removeOneshotOption(rg->getOption());
+  removeOneshotOption(option);
   return rg;
 }
 } // namespace
 
 #if defined ENABLE_BITTORRENT || ENABLE_METALINK
 namespace {
-SharedHandle<MetadataInfo> createMetadataInfo(const std::string& uri)
+std::shared_ptr<MetadataInfo> createMetadataInfo
+(const std::shared_ptr<GroupId>& gid, const std::string& uri)
 {
-  return SharedHandle<MetadataInfo>(new MetadataInfo(uri));
+  return std::make_shared<MetadataInfo>(gid, uri);
 }
 } // namespace
 
 namespace {
-SharedHandle<MetadataInfo> createMetadataInfoDataOnly()
+std::shared_ptr<MetadataInfo> createMetadataInfoDataOnly()
 {
-  return SharedHandle<MetadataInfo>(new MetadataInfo());
+  return std::make_shared<MetadataInfo>();
 }
 } // namespace
 #endif // ENABLE_BITTORRENT || ENABLE_METALINK
@@ -267,85 +181,113 @@ SharedHandle<MetadataInfo> createMetadataInfoDataOnly()
 #ifdef ENABLE_BITTORRENT
 
 namespace {
-SharedHandle<RequestGroup>
-createBtRequestGroup(const std::string& torrentFilePath,
-                     const SharedHandle<Option>& option,
+std::shared_ptr<RequestGroup>
+createBtRequestGroup(const std::string& metaInfoUri,
+                     const std::shared_ptr<Option>& optionTemplate,
                      const std::vector<std::string>& auxUris,
-                     const std::string& torrentData = "",
+                     const ValueBase* torrent,
                      bool adjustAnnounceUri = true)
 {
-  SharedHandle<RequestGroup> rg(new RequestGroup(option));
-  SharedHandle<DownloadContext> dctx(new DownloadContext());
-  if(torrentData.empty()) {
-    // may throw exception
-    bittorrent::load(torrentFilePath, dctx, option, auxUris);
-    rg->setMetadataInfo(createMetadataInfo(torrentFilePath));
-  } else {
-    // may throw exception
-    bittorrent::loadFromMemory(torrentData, dctx, option, auxUris, "default");
+  auto option = util::copy(optionTemplate);
+  auto gid = getGID(option);
+  auto rg = std::make_shared<RequestGroup>(gid, option);
+  auto dctx = std::make_shared<DownloadContext>();
+  // may throw exception
+  bittorrent::loadFromMemory(torrent, dctx, option, auxUris,
+                             metaInfoUri.empty() ? "default" : metaInfoUri);
+  if(metaInfoUri.empty()) {
     rg->setMetadataInfo(createMetadataInfoDataOnly());
+  } else {
+    rg->setMetadataInfo(createMetadataInfo(gid, metaInfoUri));
   }
   if(adjustAnnounceUri) {
     bittorrent::adjustAnnounceUri(bittorrent::getTorrentAttrs(dctx), option);
   }
-  dctx->setFileFilter(util::parseIntRange(option->get(PREF_SELECT_FILE)));
+  SegList<int> sgl;
+  util::parseIntSegments(sgl, option->get(PREF_SELECT_FILE));
+  sgl.normalize();
+  dctx->setFileFilter(sgl);
   std::istringstream indexOutIn(option->get(PREF_INDEX_OUT));
-  std::map<size_t, std::string> indexPathMap =
-    util::createIndexPathMap(indexOutIn);
-  for(std::map<size_t, std::string>::const_iterator i = indexPathMap.begin(),
-        eoi = indexPathMap.end(); i != eoi; ++i) {
+  auto indexPaths = util::createIndexPaths(indexOutIn);
+  for(const auto& i : indexPaths) {
     dctx->setFilePathWithIndex
-      ((*i).first, util::applyDir(option->get(PREF_DIR), (*i).second));
+      (i.first, util::applyDir(option->get(PREF_DIR), i.second));
   }
   rg->setDownloadContext(dctx);
   rg->setPauseRequested(option->getAsBool(PREF_PAUSE));
   // Remove "metalink" from Accept Type list to avoid server from
   // responding Metalink file for web-seeding URIs.
-  util::removeMetalinkContentTypes(rg);
-  removeOneshotOption(rg->getOption());
+  dctx->setAcceptMetalink(false);
+  removeOneshotOption(option);
   return rg;
 }
 } // namespace
 
 namespace {
-SharedHandle<RequestGroup>
-createBtMagnetRequestGroup(const std::string& magnetLink,
-                           const SharedHandle<Option>& option)
+std::shared_ptr<RequestGroup>
+createBtMagnetRequestGroup
+(const std::string& magnetLink,
+ const std::shared_ptr<Option>& optionTemplate)
 {
-  SharedHandle<RequestGroup> rg(new RequestGroup(option));
-  SharedHandle<DownloadContext> dctx
-    (new DownloadContext(METADATA_PIECE_SIZE, 0,
-                         A2STR::NIL));
+  auto option = util::copy(optionTemplate);
+  auto gid = getGID(option);
+  auto rg = std::make_shared<RequestGroup>(gid, option);
+  auto dctx = std::make_shared<DownloadContext>(METADATA_PIECE_SIZE, 0,
+                                                A2STR::NIL);
   // We only know info hash. Total Length is unknown at this moment.
   dctx->markTotalLengthIsUnknown();
   rg->setFileAllocationEnabled(false);
   rg->setPreLocalFileCheckEnabled(false);
   bittorrent::loadMagnet(magnetLink, dctx);
-  SharedHandle<TorrentAttribute> torrentAttrs =
-    bittorrent::getTorrentAttrs(dctx);
-  bittorrent::adjustAnnounceUri(torrentAttrs, rg->getOption());
+  auto torrentAttrs = bittorrent::getTorrentAttrs(dctx);
+  bittorrent::adjustAnnounceUri(torrentAttrs, option);
   dctx->getFirstFileEntry()->setPath(torrentAttrs->name);
   rg->setDownloadContext(dctx);
   rg->clearPostDownloadHandler();
-  SharedHandle<UTMetadataPostDownloadHandler> utMetadataPostHandler
-    (new UTMetadataPostDownloadHandler());
+  auto utMetadataPostHandler =
+    std::make_shared<UTMetadataPostDownloadHandler>();
   rg->addPostDownloadHandler(utMetadataPostHandler);
-  rg->setDiskWriterFactory
-    (SharedHandle<DiskWriterFactory>(new ByteArrayDiskWriterFactory()));
-  rg->setMetadataInfo(createMetadataInfo(magnetLink));
+  rg->setDiskWriterFactory(std::make_shared<ByteArrayDiskWriterFactory>());
+  rg->setMetadataInfo(createMetadataInfo(gid, magnetLink));
   rg->markInMemoryDownload();
   rg->setPauseRequested(option->getAsBool(PREF_PAUSE));
-  removeOneshotOption(rg->getOption());
+  removeOneshotOption(option);
   return rg;
 }
 } // namespace
 
 void createRequestGroupForBitTorrent
-(std::vector<SharedHandle<RequestGroup> >& result,
- const SharedHandle<Option>& option,
+(std::vector<std::shared_ptr<RequestGroup>>& result,
+ const std::shared_ptr<Option>& option,
  const std::vector<std::string>& uris,
+ const std::string& metaInfoUri,
  const std::string& torrentData,
  bool adjustAnnounceUri)
+{
+  std::unique_ptr<ValueBase> torrent;
+  bittorrent::ValueBaseBencodeParser parser;
+  if(torrentData.empty()) {
+    torrent = parseFile(parser, metaInfoUri);
+  } else {
+    ssize_t error;
+    torrent = parser.parseFinal(torrentData.c_str(), torrentData.size(),
+                                error);
+  }
+  if(!torrent) {
+    throw DL_ABORT_EX2("Bencode decoding failed",
+                       error_code::BENCODE_PARSE_ERROR);
+  }
+  createRequestGroupForBitTorrent(result, option, uris, metaInfoUri,
+                                  torrent.get());
+}
+
+void createRequestGroupForBitTorrent
+(std::vector<std::shared_ptr<RequestGroup>>& result,
+ const std::shared_ptr<Option>& option,
+ const std::vector<std::string>& uris,
+ const std::string& metaInfoUri,
+ const ValueBase* torrent,
+ bool adjustAnnounceUri)
 {
   std::vector<std::string> nargs;
   if(option->get(PREF_PARAMETERIZED_URI) == A2_V_TRUE) {
@@ -355,9 +297,8 @@ void createRequestGroupForBitTorrent
   }
   // we ignore -Z option here
   size_t numSplit = option->getAsInt(PREF_SPLIT);
-  SharedHandle<RequestGroup> rg =
-    createBtRequestGroup(option->get(PREF_TORRENT_FILE), option, nargs,
-                         torrentData, adjustAnnounceUri);
+  auto rg = createBtRequestGroup(metaInfoUri, option, nargs,
+                                 torrent, adjustAnnounceUri);
   rg->setNumConcurrentCommand(numSplit);
   result.push_back(rg);
 }
@@ -366,8 +307,8 @@ void createRequestGroupForBitTorrent
 
 #ifdef ENABLE_METALINK
 void createRequestGroupForMetalink
-(std::vector<SharedHandle<RequestGroup> >& result,
- const SharedHandle<Option>& option,
+(std::vector<std::shared_ptr<RequestGroup>>& result,
+ const std::shared_ptr<Option>& option,
  const std::string& metalinkData)
 {
   if(metalinkData.empty()) {
@@ -376,7 +317,7 @@ void createRequestGroupForMetalink
                                      option,
                                      option->get(PREF_METALINK_BASE_URI));
   } else {
-    SharedHandle<ByteArrayDiskWriter> dw(new ByteArrayDiskWriter());
+    auto dw = std::make_shared<ByteArrayDiskWriter>();
     dw->setString(metalinkData);
     Metalink2RequestGroup().generate(result, dw, option,
                                      option->get(PREF_METALINK_BASE_URI));
@@ -387,14 +328,14 @@ void createRequestGroupForMetalink
 namespace {
 class AccRequestGroup {
 private:
-  std::vector<SharedHandle<RequestGroup> >& requestGroups_;
+  std::vector<std::shared_ptr<RequestGroup>>& requestGroups_;
   ProtocolDetector detector_;
-  SharedHandle<Option> option_;
+  std::shared_ptr<Option> option_;
   bool ignoreLocalPath_;
   bool throwOnError_;
 public:
-  AccRequestGroup(std::vector<SharedHandle<RequestGroup> >& requestGroups,
-                  const SharedHandle<Option>& option,
+  AccRequestGroup(std::vector<std::shared_ptr<RequestGroup>>& requestGroups,
+                  const std::shared_ptr<Option>& option,
                   bool ignoreLocalPath = false,
                   bool throwOnError = false):
     requestGroups_(requestGroups), option_(option),
@@ -413,19 +354,23 @@ public:
       for(size_t i = 0; i < num; ++i) {
         streamURIs.push_back(uri);
       }
-      SharedHandle<RequestGroup> rg = createRequestGroup(option_, streamURIs);
+      auto rg = createRequestGroup(option_, streamURIs);
       rg->setNumConcurrentCommand(numSplit);
       requestGroups_.push_back(rg);
     }
 #ifdef ENABLE_BITTORRENT
     else if(detector_.guessTorrentMagnet(uri)) {
-      SharedHandle<RequestGroup> group =
-        createBtMagnetRequestGroup(uri, option_);
-      requestGroups_.push_back(group);
+      requestGroups_.push_back(createBtMagnetRequestGroup(uri, option_));
     } else if(!ignoreLocalPath_ && detector_.guessTorrentFile(uri)) {
       try {
+        bittorrent::ValueBaseBencodeParser parser;
+        auto torrent = parseFile(parser, uri);
+        if(!torrent) {
+          throw DL_ABORT_EX2("Bencode decoding failed",
+                             error_code::BENCODE_PARSE_ERROR);
+        }
         requestGroups_.push_back
-          (createBtRequestGroup(uri, option_, std::vector<std::string>()));
+          (createBtRequestGroup(uri, option_, {}, torrent.get()));
       } catch(RecoverableException& e) {
         if(throwOnError_) {
           throw;
@@ -435,7 +380,7 @@ public:
           A2_LOG_ERROR_EX(EX_EXCEPTION_CAUGHT, e);
         }
       }
-    } 
+    }
 #endif // ENABLE_BITTORRENT
 #ifdef ENABLE_METALINK
     else if(!ignoreLocalPath_ && detector_.guessMetalinkFile(uri)) {
@@ -476,8 +421,8 @@ public:
 } // namespace
 
 void createRequestGroupForUri
-(std::vector<SharedHandle<RequestGroup> >& result,
- const SharedHandle<Option>& option,
+(std::vector<std::shared_ptr<RequestGroup>>& result,
+ const std::shared_ptr<Option>& option,
  const std::vector<std::string>& uris,
  bool ignoreForceSequential,
  bool ignoreLocalPath,
@@ -489,95 +434,120 @@ void createRequestGroupForUri
   } else {
     nargs = uris;
   }
-  if(!ignoreForceSequential && option->get(PREF_FORCE_SEQUENTIAL) == A2_V_TRUE) {
-    std::for_each(nargs.begin(), nargs.end(),
+  if(!ignoreForceSequential &&
+     option->get(PREF_FORCE_SEQUENTIAL) == A2_V_TRUE) {
+    std::for_each(std::begin(nargs), std::end(nargs),
                   AccRequestGroup(result, option, ignoreLocalPath,
                                   throwOnError));
   } else {
-    std::vector<std::string>::iterator strmProtoEnd =
-      std::stable_partition(nargs.begin(), nargs.end(), StreamProtocolFilter());
+    auto strmProtoEnd =
+      std::stable_partition(std::begin(nargs), std::end(nargs),
+                            StreamProtocolFilter());
     // let's process http/ftp protocols first.
-    if(nargs.begin() != strmProtoEnd) {
+    if(std::begin(nargs) != strmProtoEnd) {
       size_t numIter = option->getAsInt(PREF_MAX_CONNECTION_PER_SERVER);
       size_t numSplit = option->getAsInt(PREF_SPLIT);
       std::vector<std::string> streamURIs;
-      splitURI(streamURIs, nargs.begin(), strmProtoEnd, numSplit, numIter);
-      SharedHandle<RequestGroup> rg =
-        createRequestGroup(option, streamURIs, true);
-      rg->setNumConcurrentCommand(numSplit);
-      result.push_back(rg);
+      splitURI(streamURIs, std::begin(nargs), strmProtoEnd, numSplit, numIter);
+      try {
+        auto rg = createRequestGroup(option, streamURIs, true);
+        rg->setNumConcurrentCommand(numSplit);
+        result.push_back(rg);
+      } catch(RecoverableException& e) {
+        if(throwOnError) {
+          throw;
+        } else {
+          A2_LOG_ERROR_EX(EX_EXCEPTION_CAUGHT, e);
+        }
+      }
     }
     // process remaining URIs(local metalink, BitTorrent files)
-    std::for_each(strmProtoEnd, nargs.end(),
+    std::for_each(strmProtoEnd, std::end(nargs),
                   AccRequestGroup(result, option, ignoreLocalPath,
                                   throwOnError));
   }
 }
 
-namespace {
-void createRequestGroupForUriList
-(std::vector<SharedHandle<RequestGroup> >& result,
- const SharedHandle<Option>& option,
- const std::string& filename)
+bool createRequestGroupFromUriListParser
+(std::vector<std::shared_ptr<RequestGroup>>& result,
+ const Option* option,
+ UriListParser* uriListParser)
 {
-  UriListParser p(filename);
-  while(p.hasNext()) {
+  // Since result already contains some entries, we cache the size of
+  // it. Later, we use this value to determine RequestGroup is
+  // actually created.
+  size_t num = result.size();
+  while(uriListParser->hasNext()) {
     std::vector<std::string> uris;
-    SharedHandle<Option> tempOption(new Option());
-    p.parseNext(uris, *tempOption.get());
+    Option tempOption;
+    uriListParser->parseNext(uris, tempOption);
     if(uris.empty()) {
       continue;
     }
-
-    SharedHandle<Option> requestOption(new Option(*option.get()));
+    auto requestOption = std::make_shared<Option>(*option);
     requestOption->remove(PREF_OUT);
-    for(std::set<std::string>::const_iterator i =
-          listRequestOptions().begin(), eoi = listRequestOptions().end();
-        i != eoi; ++i) {
-      if(tempOption->defined(*i)) {
-        requestOption->put(*i, tempOption->get(*i));
+    const auto& oparser = OptionParser::getInstance();
+    for(size_t i = 1, len = option::countOption(); i < len; ++i) {
+      auto pref = option::i2p(i);
+      auto h = oparser->find(pref);
+      if(h && h->getInitialOption() && tempOption.defined(pref)) {
+        requestOption->put(pref, tempOption.get(pref));
       }
     }
-
+    // This does not throw exception because throwOnError = false.
     createRequestGroupForUri(result, requestOption, uris);
+    if(num < result.size()) {
+      return true;
+    }
   }
+  return false;
+}
+
+std::shared_ptr<UriListParser> openUriListParser(const std::string& filename)
+{
+  std::string listPath;
+  if(filename == "-") {
+    listPath = DEV_STDIN;
+  } else {
+    if(!File(filename).isFile()) {
+      throw DL_ABORT_EX
+        (fmt(EX_FILE_OPEN, filename.c_str(), "No such file"));
+    }
+    listPath = filename;
+  }
+  return std::make_shared<UriListParser>(listPath);
 }
-} // namespace
 
 void createRequestGroupForUriList
-(std::vector<SharedHandle<RequestGroup> >& result,
- const SharedHandle<Option>& option)
+(std::vector<std::shared_ptr<RequestGroup>>& result,
+ const std::shared_ptr<Option>& option)
 {
-  if(option->get(PREF_INPUT_FILE) == "-") {
-    createRequestGroupForUriList(result, option, DEV_STDIN);
-  } else {
-    if(!File(option->get(PREF_INPUT_FILE)).isFile()) {
-      throw DL_ABORT_EX
-        (fmt(EX_FILE_OPEN, option->get(PREF_INPUT_FILE).c_str(),
-             "No such file"));
-    }
-    createRequestGroupForUriList(result, option, option->get(PREF_INPUT_FILE));
-  }
+  auto uriListParser = openUriListParser(option->get(PREF_INPUT_FILE));
+  while(createRequestGroupFromUriListParser(result, option.get(),
+                                            uriListParser.get()));
 }
 
-SharedHandle<MetadataInfo>
-createMetadataInfoFromFirstFileEntry(const SharedHandle<DownloadContext>& dctx)
+std::shared_ptr<MetadataInfo>
+createMetadataInfoFromFirstFileEntry
+(const std::shared_ptr<GroupId>& gid,
+ const std::shared_ptr<DownloadContext>& dctx)
 {
   if(dctx->getFileEntries().empty()) {
-    return SharedHandle<MetadataInfo>();
+    return nullptr;
   } else {
     std::vector<std::string> uris;
     dctx->getFileEntries()[0]->getUris(uris);
     if(uris.empty()) {
-      return SharedHandle<MetadataInfo>();
+      return nullptr;
     }
-    return SharedHandle<MetadataInfo>(new MetadataInfo(uris[0]));
+    return std::make_shared<MetadataInfo>(gid, uris[0]);
   }
 }
 
-void removeOneshotOption(const SharedHandle<Option>& option)
+void removeOneshotOption(const std::shared_ptr<Option>& option)
 {
   option->remove(PREF_PAUSE);
+  option->remove(PREF_GID);
 }
 
 } // namespace aria2
